###########################################################
## This is an R file that forms part of the research
## project "An Exponential Chi-Squared QMLE for Log-GARCH
## Models Via the ARMA Representation" by Christian Francq
## and Genaro Sucarrat
##
## First created: 17 June 2014, Madrid
## Last changed: 21 August 2017, Oslo
##
## Contents:
##
## 1 INITIATE
## 2 AUXILIARY FUNCTIONS
## 3 FRANCQ, WINTENBERGER AND ZAKOIAN (2013) LOG-GARCH
## 4 COMBINED ESTIMATOR CODE
## 5 SP500 RETURNS
## 6 APPLE DATA
## 7 EKORNES RETURNS
## 8 TABLES AND GRAPHS

##
###########################################################

###########################################################
##1 INITIATE
###########################################################

#set working directory:
setwd(choose.dir())

#lgarch code version 0.4 (uncompiled):
source("20140701lgarch.Rpackage.R")

##zero-adjusted garch models:
source("garchzero_CODE_V01.R")

##load required libraries:
library(xtable)
library(compiler)
require(MASS)


###########################################################
##2 AUXILIARY FUNCTIONS
###########################################################

##================================================
##rounding as taught in many elementary schools:
ground <- function(x, digits=0, eps=.Machine$double.eps,
  as.character=FALSE)
{
xpos <- as.numeric(x>0)*(x + eps)
xneg <- as.numeric(x<0)*(x - eps)
x <- xpos + xneg
if(as.character){
  result <- format(round(x, digits=digits), nsmall=digits)
}else{
  result <- round(x, digits=digits)
}
return(result)
} #end ground

##================================================
#Sample kurtosis
gkurtosis <- function(y)
{
m4 <- mean((y-mean(y))^4)
kurt <- m4/(sd(y)^4)
return(kurt)
} #end gkurtosis

##================================================
#Sample skewness
gskewness <- function(y)
{
m3 <- mean((y-mean(y))^3)
skew <- m3/(sd(y)^3)
return(skew)
} #end gskewness

##================================================
#Ljung-Box test:
gBox <- function(x, lag=12, zeros.as.na=FALSE)
{
  #check/modify arguments:
  if(NCOL(x)>1){ stop("x is not a vector or univariate time series") }
  if(lag < 1){ stop("lag must be equal to or greater than 1") }
  x <- as.numeric(x)
  if(zeros.as.na){ x[x==0] <- NA }

  #compute:
  result <- matrix(NA, max(lag), 5)
  #colnames(result) <- c("AutoCorr", "Q-stat", "P-val", "Df", "Obs")
  for(i in 1:max(lag)){

    #preliminaries:
    mX <- cbind(x, glag(x, k=i, pad=TRUE, pad.value=NA))
    isna <- is.na(mX)
    rownotna <- rowSums(isna) == 0
    result[i,1] <- cor(mX[rownotna,1], mX[rownotna,2])

    #Ljung-Box test:
    result[i,5] <- sum(rownotna)
    n <- result[i,5]
    statval <- n*(n+2)*sum(result[c(1:i),1]^2)/(n-i)
    result[i,2] <- statval
    pval <- 1 - pchisq(statval,i)
    result[i,3] <- pval
    result[i,4] <- i

  } #end for(..)

  #out:
  colnames(result) <- c("AutoCorr", "Q-stat", "P-val", "Df", "Obs")
  rownames(result) <- 1:lag
  return(result)
} #end gBox


###########################################################
##3 FRANCQ, WINTENBERGER AND ZAKOIAN (2013) LOG-GARCH
###########################################################

#lgarchf recursion (R code):
lgarchf.recursionR <- function(n, logsigma2, beta1, innov){
  for(i in 1:n){ logsigma2[i+1] <- beta1*logsigma2[i] + innov[i] }
  return(logsigma2)
}
lgarchf.recursionR <- cmpfun(lgarchf.recursionR)

#lgarchf simulation and recursion (C code):
mysig <- signature(n="integer", logsigma2="numeric",
  beta1="numeric", innov="numeric")
mycode <- "
  for (int i=0; i < *n; i++) {
    logsigma2[i+1] = *beta1 * logsigma2[i] + innov[i];
  }
"
lgarchf.recursionC <- cfunction(mysig, mycode, convention=".C") #compile
rm(mysig, mycode)

##===============
lgarchf.recursion <- function(pars, aux)
{
if(is.null(aux$Elogz2)){
  Elogz2 <- (aux$Elogy2*(1-pars[2]-pars[3]) - pars[1])/(1-pars[3])
}else{
  Elogz2 <- aux$Elogz2
}
logsigma2 <- rep(0,aux$npluss1)
if(is.null(aux$logsigma2.initial)){
  logsigma2[1] <- (pars[1]+pars[2]*Elogz2)/(1-pars[2]-pars[3])
}else{
  logsigma2[1] <- aux$logsigma2.initial
}
logy2adj <- c(logsigma2[1]+Elogz2, aux$logy2adj)

if(aux$c.code){
  innov <- pars[1] + pars[2]*logy2adj
  tmp <- lgarchf.recursionC(as.numeric(aux$n),
    as.numeric(logsigma2), as.numeric(pars[3]),
    as.numeric(innov))
  logsigma2 <- tmp$logsigma2
}else{
#  for(i in 1:aux$n){ logsigma2[i+1] <- pars[1] + pars[2]*logy2adj[i] + pars[3]*logsigma2[i] }
  innov <- pars[1] + pars[2]*logy2adj
  logsigma2 <- lgarchf.recursionR(n, logsigma2, pars[3], innov)
}

#output:
logsigma2 <- logsigma2[-1]
if(aux$verbose){
  sigma2 <- exp(logsigma2)
  sigma <- sqrt(sigma2)
  z <- aux$y/sigma
  logy2adj <- logy2adj[-1]
  result <- cbind(sigma, sigma2, logsigma2, logy2adj, z)
}else{
  result <- logsigma2
}
return(result)
} #end lgarchf.recursion

##===============
lgarchf.logl <- function(pars, aux=NULL)
{
if( any(is.na(pars)) || any(pars<=aux$lower) || any(pars>=aux$upper) ){
  chk.conds <- FALSE
}else{
  chk.conds <- TRUE
}

if(chk.conds){
  logsigma2 <- lgarchf.recursion(pars, aux)
  sigma2 <- exp(logsigma2)
  if( any(is.na(sigma2)) ){
    logl <- logl.penalty
  }else{
    term1 <- -aux$n*log(2*pi)
    term2 <- -sum(logsigma2)/2
    term3 <- -sum(aux$y2/sigma2)/2

    logl <- term1 + term2 + term3
  }
  if( is.na(logl) || abs(logl) == Inf ){ logl <- aux$logl.penalty }
}else{ logl <- aux$logl.penalty }

return(logl)
} #lgarchf.logl

##===============
lgarchf.est <- function(y, initial.values=c(0.001,0.05,0.9),
  lower=c(-Inf,-1+.Machine$double.eps,-1+.Machine$double.eps),
  upper=c(Inf,1-.Machine$double.eps,1-.Machine$double.eps),
  hessian=TRUE, logsigma2.initial=NULL, Elogz2=NULL,
  c.code=TRUE, verbose=TRUE, logl.penalty=NULL, ...)
{
#create aux list:
aux <- NULL
aux$n <- length(y)
aux$npluss1 <- aux$n + 1
aux$wherezero <- which(y==0)
aux$y <- y
aux$y2 <- y^2
if(length(aux$wherezero)>0){
  yadj <- y
  yadj[aux$wherezero] <- 1
  aux$logy2adj <- log(yadj^2)
  aux$Elogy2 <- mean(aux$logy2adj[-aux$wherezero])
}else{
  aux$logy2adj <- log(y^2)
  aux$Elogy2 <- mean(aux$logy2)
}
aux$lower <- lower
aux$upper <- upper
aux$Elogz2 <- Elogz2
aux$logsigma2.initial <- logsigma2.initial
aux$c.code <- c.code
aux$verbose <- FALSE
if(is.null(logl.penalty)){
  aux$logl.penalty <- lgarchf.logl(initial.values, aux)
}else{ aux$logl.penalty <- logl.penalty }

objective.f <- function(pars, aux=aux){ -lgarchf.logl(pars,
  aux=aux) }
est <- nlminb(initial.values, objective.f, lower=lower,
  upper=upper, aux=aux, ...)
est$objective <- -est$objective

#compute Hessian:
if(hessian){
  hess <- -optimHess(est$par, objective.f, aux=aux)
  est <- c(list(hessian=hess), est)
}

if(verbose){
  parnames <- c("constant", "arch", "garch")
  names(est$par) <- parnames
  if(hessian){
    colnames(est$hessian) <- parnames
    rownames(est$hessian) <- parnames
  }
  names(initial.values) <- parnames
  names(lower) <- parnames
  names(upper) <- parnames
  est <- c(list(date=date(), initial.values=initial.values,
    lower=lower, upper=upper), est)
}

#out:
if(is.null(est$aux)){
  est <- c(list(aux=aux),est)
}
return(est)
} #end lgarchf.est

##===============
lgarchf.fit <- function(pars, aux)
{
aux$verbose <- TRUE
result <- lgarchf.recursion(pars, aux)
return(result)
} #end lgarchf.fit


###########################################################
##4 COMBINED ESTIMATOR CODE
###########################################################

#weights
wghts <- function(eta)
{
  #where zeros:
  where0 <- which(eta==0)
  if(length(where0)>0){ eta <- eta[-where0] }

  #mSigmaEtaHat entries
  eta2 <- eta^2
  vareta2 <- var(eta2)
  lneta2 <- log(eta2)
  varlneta2 <- var(lneta2)
  coveta2lneta2 <- cov(eta2, lneta2)

  #fill mSigmaEtaHat:
  mSigmaEtaHat <- matrix(NA,2,2)
  vOnes <- cbind(c(1,1))
  mSigmaEtaHat[1,1] <- vareta2
  mSigmaEtaHat[1,2] <- coveta2lneta2
  mSigmaEtaHat[2,1] <- coveta2lneta2
  mSigmaEtaHat[2,2] <- varlneta2

  #compute weights:
  mSigmaEtaHatInv <- ginv(mSigmaEtaHat)
  omegac <- (mSigmaEtaHatInv %*% vOnes)/as.numeric(t(vOnes) %*% mSigmaEtaHatInv %*% vOnes)
  colnames(omegac) <- "omegac"

  return(omegac)
} #end wghts

#estimate of asymptotic Cov[sqrt(n)*(estimate - true)]:
vcovhat <- function(coefs, method, etahat)
{
  alpha0 <- as.numeric(coefs[2])
  beta0 <- as.numeric(coefs[3])
  where0 <- which(etahat==0)
  n0 <- length(where0)
  nstar <- length(etahat) - n0

  #common part:
  mTmp <- matrix(NA,2,2)
  mTmp[1,1] <- 1-beta0^2 * (alpha0+beta0)^2
  mTmp[2,1] <- ((alpha0+beta0)*(1-beta0^2)*(1-beta0*(alpha0+beta0)))/(alpha0^2)
  mTmp[1,2] <- mTmp[2,1]
  mTmp[2,2] <- (1-beta0^2)*(1-beta0*(alpha0+beta0))^2/(alpha0^2)

  #compute avcov:
  if(method %in% c("ml","ls")){
    #Gaussian ARMA-QMLE:
    vcovhat <- mTmp/nstar
  }else{
    #cex2, standard QMLE:
    if(n0>0){ etahat <- etahat[-where0] }
    vareta2 <- var(etahat^2)
    varlneta2 <- var(log(etahat^2))
    vcovhat <- (vareta2/varlneta2)*mTmp/nstar
  }

  return(vcovhat)
} #end vcovhat

#combined avar estimate:
avarc <- function(eta, coef1, coef2)
{
  #where zeros:
  where0 <- which(eta==0)
  if(length(where0)>0){ eta <- eta[-where0] }

  #mSigmaEtaHat entries
  eta2 <- eta^2
  vareta2 <- var(eta2)
  lneta2 <- log(eta2)
  varlneta2 <- var(lneta2)
  coveta2lneta2 <- cov(eta2, lneta2)

  #mSigmaEtaHat:
  mSigmaEtaHat <- matrix(NA,2,2)
  mSigmaEtaHat[1,1] <- vareta2
  mSigmaEtaHat[1,2] <- coveta2lneta2
  mSigmaEtaHat[2,1] <- coveta2lneta2
  mSigmaEtaHat[2,2] <- varlneta2
  #mSigmaEtaHatInv <- ginv(mSigmaEtaHat)

  #combined estimates:
  omegac <- wghts(eta)
  alpha0 <- t(omegac) %*% cbind(c(coef1["arch1"],coef2["arch1"]))
  alpha0 <- as.numeric(alpha0)
  beta0 <- t(omegac) %*% cbind(c(coef1["garch1"],coef2["garch1"]))
  beta0 <- as.numeric(beta0)

  #mSigmauInv:
  mTmp <- matrix(NA,2,2)
  mTmp[1,1] <- 1-beta0^2 * (alpha0+beta0)^2
  mTmp[2,1] <- ((alpha0+beta0)*(1-beta0^2)*(1-beta0*(alpha0+beta0)))/(alpha0^2)
  mTmp[1,2] <- mTmp[2,1]
  mTmp[2,2] <- (1-beta0^2)*(1-beta0*(alpha0+beta0))^2/(alpha0^2)
  mSigmauInv <- mTmp/varlneta2

  #combined variance:
  avarc <- as.numeric(t(omegac) %*% mSigmaEtaHat %*% omegac) * mSigmauInv
  avarc <- diag(avarc)

  return(avarc)
} #end avarc

#recursion:
lgarchRecursion <- function(y, omega0, alpha0, beta0, Elneta2)
{
  #prepare recursion:
  yzero <- as.numeric(y != 0)
  yadj <- y
  where0 <- which(y==0)
  if(length(where0)>0){ yadj[where0] <- 1 }
  lnyadj2 <- log(yadj^2)
  
  #log-garch(1,1) recursion:
  npluss1 <- length(y)+1
  logsigma2 <- rep(NA, npluss1)
  logsigma2[1] <- (omega0+alpha0*Elneta2)/(1-alpha0-beta0)
  lnyadj2 <- c( logsigma2[1]+Elneta2, lnyadj2)
  yzeroadj <- c(1,yzero)

  for(i in 2:npluss1){
    logsigma2[i] <- omega0 + alpha0*lnyadj2[i-1] + beta0*logsigma2[i-1]
    if(yzeroadj[i]==0){ lnyadj2[i] <- logsigma2[i] + Elneta2 }
  }

  #out:
  out <- list()
  out$logsigma2 <- logsigma2[-1]
  out$sigma2 <- exp(out$logsigma2)
  out$etahat <- y/sqrt(out$sigma2)
  
  return(out)
} #end lgarchRecursion


###########################################################
##5 SP500 RETURNS
###########################################################

mSP500 <- read.csv("sp500.csv")
sp500 <- zoo(mSP500[,"Adj.Close"], order.by=as.Date(mSP500[,1], "%Y-%m-%d"))
Dlsp500 <- diff(log(sp500))*100
t1 <- which(index(Dlsp500) == "1999-01-04")
t2 <- which(index(Dlsp500) == "2013-08-23")
Dlsp500 <- Dlsp500[t1:t2]

#zeros replaced by small non-zero value:
Dlsp500.where.zeros <- which( Dlsp500 == 0)
Dlsp500.zero.eps <- min(abs(Dlsp500[-Dlsp500.where.zeros]))
Dlsp500adj <- Dlsp500
Dlsp500adj[Dlsp500.where.zeros] <- Dlsp500.zero.eps

#cex2 models (zeros replaced w/small number):
Dlsp500.Mod01 <- lgarch(Dlsp500adj, method="cex2")
Dlsp500.Mod01Fit <- fitted(Dlsp500.Mod01, verbose=TRUE)
Dlsp500.Mod01LogL0 <- logLik(Dlsp500.Mod01)/length(Dlsp500adj)
Dlsp500.Mod01LogL <- sum(dnorm(Dlsp500[Dlsp500!=0],
  sd=Dlsp500.Mod01Fit[Dlsp500!=0,1], log=TRUE))/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod01AR <- gBox(Dlsp500.Mod01Fit[,"zhat"], lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod01ARCH <- gBox(Dlsp500.Mod01Fit[,"zhat"]^2, lag=12,
  zeros.as.na=FALSE)

#cex2-em models (zeros kept):
Dlsp500.Mod02 <- lgarch(Dlsp500, method="cex2")
Dlsp500.Mod02Fit <- fitted(Dlsp500.Mod02, verbose=TRUE)
Dlsp500.Mod02LogL <- logLik(Dlsp500.Mod02)/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod02AR <- gBox(Dlsp500.Mod02Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlsp500.Mod02ARCH <- gBox(Dlsp500.Mod02Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlsp500.Mod02etahat <- residuals(Dlsp500.Mod02)
Dlsp500.Mod02VCOV <- vcovhat(coef(Dlsp500.Mod02), "cex2",
  Dlsp500.Mod02etahat)
Dlsp500.Mod02SEs <- sqrt(diag(Dlsp500.Mod02VCOV))

#Francq et al. QMLE:
Dlsp500.Mod03 <- lgarchf.est(as.numeric(Dlsp500))
Dlsp500.Mod03Fit <- lgarchf.fit(as.numeric(Dlsp500.Mod03$par),
  Dlsp500.Mod03$aux)
Dlsp500.Mod03Fit <- zoo(Dlsp500.Mod03Fit,
  order.by=index(Dlsp500))
Dlsp500.Mod03LogL0 <- Dlsp500.Mod03$objective/length(Dlsp500)
Dlsp500.Mod03LogL <- sum(dnorm(Dlsp500[Dlsp500!=0],
  sd=Dlsp500.Mod03Fit[Dlsp500!=0,1], log=TRUE))/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod03AR <- gBox(Dlsp500.Mod03Fit[,"z"], lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod03ARCH <- gBox(Dlsp500.Mod03Fit[,"z"]^2, lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod03etahat <- as.numeric(Dlsp500.Mod03Fit[,"z"])
Dlsp500.Mod03VCOV <- vcovhat(Dlsp500.Mod03$par, "standard",
  Dlsp500.Mod03etahat)
Dlsp500.Mod03SEs <- sqrt(diag(Dlsp500.Mod03VCOV))

#Gaussian ARMA-QMLE models, zeros kept:
Dlsp500.Mod04 <- lgarch(Dlsp500, method="ml",
  mean.correction=TRUE)
Dlsp500.Mod04Fit <- fitted(Dlsp500.Mod04, verbose=TRUE)
Dlsp500.Mod04LogL <- logLik(Dlsp500.Mod04)/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod04AR <- gBox(Dlsp500.Mod04Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlsp500.Mod04ARCH <- gBox(Dlsp500.Mod04Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlsp500.Mod04etahat <- residuals(Dlsp500.Mod04)
Dlsp500.Mod04VCOV <- vcovhat(coef(Dlsp500.Mod04), "ml",
  Dlsp500.Mod04etahat)
Dlsp500.Mod04SEs <- sqrt(diag(Dlsp500.Mod04VCOV))

#combined estimator, zeros kept:
Dlsp500.Mod05weights <- wghts(c(coredata(Dlsp500.Mod02etahat),
  coredata(Dlsp500.Mod04etahat)))
Dlsp500.Mod05omega0 <- as.numeric( t(Dlsp500.Mod05weights) %*%
  c(coef(Dlsp500.Mod02)[1],coef(Dlsp500.Mod04)[1]) )
Dlsp500.Mod05alpha0 <- as.numeric( t(Dlsp500.Mod05weights) %*%
  c(coef(Dlsp500.Mod02)[2],coef(Dlsp500.Mod04)[2]) )
Dlsp500.Mod05beta0 <- as.numeric( t(Dlsp500.Mod05weights) %*%
  c(coef(Dlsp500.Mod02)[3],coef(Dlsp500.Mod04)[3]) )
Dlsp500.Mod05mu <- as.numeric( coef(Dlsp500.Mod02)[4] + coef(Dlsp500.Mod04)[4])/2
Dlsp500.Mod05Fit <- sqrt(lgarchRecursion(as.numeric(Dlsp500),
  Dlsp500.Mod05omega0, Dlsp500.Mod05alpha0,
  Dlsp500.Mod05beta0, Dlsp500.Mod05mu)$sigma2)
Dlsp500.Mod05Fit <- zoo(Dlsp500.Mod05Fit,
  order.by=index(Dlsp500))
Dlsp500.Mod05LogL <- sum(dnorm(Dlsp500[Dlsp500!=0],
  sd=Dlsp500.Mod05Fit[Dlsp500!=0,1], log=TRUE))/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod05AVARc <- avarc(c(coredata(Dlsp500.Mod02etahat),
  coredata(Dlsp500.Mod04etahat)), coef(Dlsp500.Mod02),
  coef(Dlsp500.Mod04))
Dlsp500.Mod05SEc <- sqrt(Dlsp500.Mod05AVARc/length(Dlsp500[Dlsp500!=0]))

#ordinary GARCH:
Dlsp500.Mod06 <- garch.est(Dlsp500)
Dlsp500.Mod06Fit <- garch.fit(Dlsp500, as.numeric(Dlsp500.Mod06$par))
Dlsp500.Mod06LogL0 <- Dlsp500.Mod06$objective/length(Dlsp500)
Dlsp500.Mod06LogL <- sum(dnorm(Dlsp500[Dlsp500!=0],
  sd=Dlsp500.Mod06Fit[Dlsp500!=0,"sigma"], log=TRUE))/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod06AR <- gBox(Dlsp500.Mod06Fit[,"resid.std"], lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod06ARCH <- gBox(Dlsp500.Mod06Fit[,"resid.std"]^2, lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod06etahat <- as.numeric(Dlsp500.Mod06Fit[,"resid.std"])
Dlsp500.Mod06VCOV <- solve(-Dlsp500.Mod06$hessian)
Dlsp500.Mod06SEs <- sqrt(diag(Dlsp500.Mod06VCOV))

##zero-adjusted GARCH:
Dlsp500.Mod07 <- garch.estz(Dlsp500)
Dlsp500.Mod07Fit <- garch.fitz(Dlsp500, as.numeric(Dlsp500.Mod07$par))
Dlsp500.Mod07LogL0 <- Dlsp500.Mod07$objective/length(Dlsp500)
Dlsp500.Mod07LogL <- sum(dnorm(Dlsp500[Dlsp500!=0],
  sd=Dlsp500.Mod07Fit[Dlsp500!=0,"sd"], log=TRUE))/(length(Dlsp500)-sum(Dlsp500==0))
Dlsp500.Mod07AR <- gBox(Dlsp500.Mod07Fit[,"residuals"], lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod07ARCH <- gBox(Dlsp500.Mod07Fit[,"residuals"]^2, lag=12,
  zeros.as.na=FALSE)
Dlsp500.Mod07etahat <- as.numeric(Dlsp500.Mod07Fit[,"residuals"])
Dlsp500.Mod07VCOV <- solve(-Dlsp500.Mod07$hessian)
Dlsp500.Mod07SEs <- sqrt(diag(Dlsp500.Mod07VCOV))

#sd differences and ratios:
Dlsp500.Mod02vs03diff <- Dlsp500.Mod02Fit[,"sd"] - Dlsp500.Mod03Fit[,"sigma"]
Dlsp500.Mod02vs03ratio <- Dlsp500.Mod02Fit[,"sd"]/Dlsp500.Mod03Fit[,"sigma"]
Dlsp500.Mod02vs04diff <- Dlsp500.Mod02Fit[,"sd"] - Dlsp500.Mod04Fit[,"sd"]
Dlsp500.Mod02vs04ratio <- Dlsp500.Mod02Fit[,"sd"]/Dlsp500.Mod04Fit[,"sd"]
Dlsp500.Mod05vs03diff <- Dlsp500.Mod05Fit - Dlsp500.Mod03Fit[,"sigma"]
Dlsp500.Mod05vs03ratio <- Dlsp500.Mod05Fit/Dlsp500.Mod03Fit[,"sigma"]
Dlsp500.Mod07vs06ratio <- Dlsp500.Mod07Fit[,"sd"]/Dlsp500.Mod06Fit[,"sigma"]
#plot(Dlsp500.Mod07vs06ratio)

##desc stat of GARCH models:
mean(Dlsp500.Mod06Fit[,"sigma"])
var(Dlsp500.Mod06Fit[,"sigma"])
max(Dlsp500.Mod06Fit[,"sigma"])
min(Dlsp500.Mod06Fit[,"sigma"])
mean(Dlsp500.Mod07Fit[,"sd"])
var(Dlsp500.Mod07Fit[,"sd"])
max(Dlsp500.Mod07Fit[,"sd"])
min(Dlsp500.Mod07Fit[,"sd"])
mean(Dlsp500.Mod07vs06ratio)
var(Dlsp500.Mod07vs06ratio)
max(Dlsp500.Mod07vs06ratio)
min(Dlsp500.Mod07vs06ratio)

#latex code:
Dlsp500.latex <- paste("SP500 & & ", ground(var(Dlsp500),
  digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlsp500),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox(Dlsp500^2, lag=1)[1,"P-val"],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox(Dlsp500^2, lag=1)[1,"Q-stat"],
  digits=2, as.character=TRUE), "}$ & ",
  length(Dlsp500), " & ", length(Dlsp500.where.zeros), " & ",
  ground(length(Dlsp500.where.zeros)/length(Dlsp500), digits=3,
  as.character=TRUE), " \\", sep="")


###########################################################
##6 APPLE DATA
###########################################################

mapple <- read.csv("apple.csv")
apple <- zoo(mapple[,"Adj.Close"], order.by=as.Date(mapple[,1], "%Y-%m-%d"))
Dlapple <- diff(log(apple))*100

#zeros replaced by small non-zero value:
Dlapple.where.zeros <- which( Dlapple == 0)
Dlapple.zero.eps <- min(abs(Dlapple[-Dlapple.where.zeros]))
Dlappleadj <- Dlapple
Dlappleadj[Dlapple.where.zeros] <- Dlapple.zero.eps

#Cex2 ARMA-QMLE, zeros replaced:
Dlapple.Mod01 <- lgarch(Dlappleadj, method="cex2")
Dlapple.Mod01Fit <- fitted(Dlapple.Mod01, verbose=TRUE)
Dlapple.Mod01LogL0 <- logLik(Dlapple.Mod01)/length(Dlappleadj)
Dlapple.Mod01LogL <- sum(dnorm(Dlapple[Dlapple!=0],
  sd=Dlapple.Mod01Fit[Dlapple!=0,1], log=TRUE))/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod01AR <- gBox(Dlapple.Mod01Fit[,"zhat"], lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod01ARCH <- gBox(Dlapple.Mod01Fit[,"zhat"]^2, lag=12,
  zeros.as.na=FALSE)

#Cex2 ARMA-QMLE, zeros kept:
Dlapple.Mod02 <- lgarch(Dlapple, method="cex2")
Dlapple.Mod02Fit <- fitted(Dlapple.Mod02, verbose=TRUE)
Dlapple.Mod02LogL <- logLik(Dlapple.Mod02)/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod02AR <- gBox(Dlapple.Mod02Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlapple.Mod02ARCH <- gBox(Dlapple.Mod02Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlapple.Mod02etahat <- residuals(Dlapple.Mod02)
Dlapple.Mod02VCOV <- vcovhat(coef(Dlapple.Mod02), "cex2",
  Dlapple.Mod02etahat)
Dlapple.Mod02SEs <- sqrt(diag(Dlapple.Mod02VCOV))

#Francq et al. QMLE:
Dlapple.Mod03 <- lgarchf.est(as.numeric(Dlapple))
Dlapple.Mod03Fit <- lgarchf.fit(as.numeric(Dlapple.Mod03$par),
  Dlapple.Mod03$aux)
Dlapple.Mod03Fit <- zoo(Dlapple.Mod03Fit,
  order.by=index(Dlapple))
Dlapple.Mod03LogL0 <- Dlapple.Mod03$objective/length(Dlapple)
Dlapple.Mod03LogL <- sum(dnorm(Dlapple[Dlapple!=0],
  sd=Dlapple.Mod03Fit[Dlapple!=0,1], log=TRUE))/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod03AR <- gBox(Dlapple.Mod03Fit[,"z"], lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod03ARCH <- gBox(Dlapple.Mod03Fit[,"z"]^2, lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod03etahat <- as.numeric(Dlapple.Mod03Fit[,"z"])
Dlapple.Mod03VCOV <- vcovhat(Dlapple.Mod03$par, "standard",
  Dlapple.Mod03etahat)
Dlapple.Mod03SEs <- sqrt(diag(Dlapple.Mod03VCOV))

#Gaussian ARMA-QMLE, zeros kept:
Dlapple.Mod04 <- lgarch(Dlapple, method="ml",
  mean.correction=TRUE)
Dlapple.Mod04Fit <- fitted(Dlapple.Mod04, verbose=TRUE)
Dlapple.Mod04LogL <- logLik(Dlapple.Mod04)/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod04AR <- gBox(Dlapple.Mod04Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlapple.Mod04ARCH <- gBox(Dlapple.Mod04Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlapple.Mod04etahat <- residuals(Dlapple.Mod04)
Dlapple.Mod04VCOV <- vcovhat(coef(Dlapple.Mod04), "ml",
  Dlapple.Mod04etahat)
Dlapple.Mod04SEs <- sqrt(diag(Dlapple.Mod04VCOV))

#combined estimator, zeros kept:
Dlapple.Mod05weights <- wghts(c(coredata(Dlapple.Mod02etahat),
  coredata(Dlapple.Mod04etahat)))
Dlapple.Mod05omega0 <- as.numeric( t(Dlapple.Mod05weights) %*%
  c(coef(Dlapple.Mod02)[1],coef(Dlapple.Mod04)[1]) )
Dlapple.Mod05alpha0 <- as.numeric( t(Dlapple.Mod05weights) %*%
  c(coef(Dlapple.Mod02)[2],coef(Dlapple.Mod04)[2]) )
Dlapple.Mod05beta0 <- as.numeric( t(Dlapple.Mod05weights) %*%
  c(coef(Dlapple.Mod02)[3],coef(Dlapple.Mod04)[3]) )
Dlapple.Mod05mu <- as.numeric( coef(Dlapple.Mod02)[4] + coef(Dlapple.Mod04)[4])/2
Dlapple.Mod05Fit <- sqrt(lgarchRecursion(as.numeric(Dlapple),
  Dlapple.Mod05omega0, Dlapple.Mod05alpha0,
  Dlapple.Mod05beta0, Dlapple.Mod05mu)$sigma2)
Dlapple.Mod05Fit <- zoo(Dlapple.Mod05Fit,
  order.by=index(Dlapple))
Dlapple.Mod05LogL <- sum(dnorm(Dlapple[Dlapple!=0],
  sd=Dlapple.Mod05Fit[Dlapple!=0,1], log=TRUE))/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod05AVARc <- avarc(c(coredata(Dlapple.Mod02etahat),
  coredata(Dlapple.Mod04etahat)), coef(Dlapple.Mod02),
  coef(Dlapple.Mod04))
Dlapple.Mod05SEc <- sqrt(Dlapple.Mod05AVARc/length(Dlapple[Dlapple!=0]))

#ordinary GARCH:
Dlapple.Mod06 <- garch.est(Dlapple)
Dlapple.Mod06Fit <- garch.fit(Dlapple, as.numeric(Dlapple.Mod06$par))
Dlapple.Mod06LogL0 <- Dlapple.Mod06$objective/length(Dlapple)
Dlapple.Mod06LogL <- sum(dnorm(Dlapple[Dlapple!=0],
  sd=Dlapple.Mod06Fit[Dlapple!=0,"sigma"], log=TRUE))/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod06AR <- gBox(Dlapple.Mod06Fit[,"resid.std"], lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod06ARCH <- gBox(Dlapple.Mod06Fit[,"resid.std"]^2, lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod06etahat <- as.numeric(Dlapple.Mod06Fit[,"resid.std"])
Dlapple.Mod06VCOV <- solve(-Dlapple.Mod06$hessian)
Dlapple.Mod06SEs <- sqrt(diag(Dlapple.Mod06VCOV))

##zero-adjusted GARCH:
Dlapple.Mod07 <- garch.estz(Dlapple)
Dlapple.Mod07Fit <- garch.fitz(Dlapple, as.numeric(Dlapple.Mod07$par))
Dlapple.Mod07LogL0 <- Dlapple.Mod07$objective/length(Dlapple)
Dlapple.Mod07LogL <- sum(dnorm(Dlapple[Dlapple!=0],
  sd=Dlapple.Mod07Fit[Dlapple!=0,"sd"], log=TRUE))/(length(Dlapple)-sum(Dlapple==0))
Dlapple.Mod07AR <- gBox(Dlapple.Mod07Fit[,"residuals"], lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod07ARCH <- gBox(Dlapple.Mod07Fit[,"residuals"]^2, lag=12,
  zeros.as.na=FALSE)
Dlapple.Mod07etahat <- as.numeric(Dlapple.Mod07Fit[,"residuals"])
Dlapple.Mod07VCOV <- solve(-Dlapple.Mod07$hessian)
Dlapple.Mod07SEs <- sqrt(diag(Dlapple.Mod07VCOV))

##sd differences and ratios:
Dlapple.Mod02vs03diff <- Dlapple.Mod02Fit[,"sd"] - Dlapple.Mod03Fit[,"sigma"]
Dlapple.Mod02vs03ratio <- Dlapple.Mod02Fit[,"sd"]/Dlapple.Mod03Fit[,"sigma"]
Dlapple.Mod02vs04diff <- Dlapple.Mod02Fit[,"sd"] - Dlapple.Mod04Fit[,"sd"]
Dlapple.Mod02vs04ratio <- Dlapple.Mod02Fit[,"sd"]/Dlapple.Mod04Fit[,"sd"]
Dlapple.Mod05vs03diff <- Dlapple.Mod05Fit - Dlapple.Mod03Fit[,"sigma"]
Dlapple.Mod05vs03ratio <- Dlapple.Mod05Fit/Dlapple.Mod03Fit[,"sigma"]
Dlapple.Mod07vs06ratio <- Dlapple.Mod07Fit[,"sd"]/Dlapple.Mod06Fit[,"sigma"]
#plot(Dlapple.Mod07vs06ratio)

##desc stat of GARCH models:
mean(Dlapple.Mod06Fit[,"sigma"])
var(Dlapple.Mod06Fit[,"sigma"])
max(Dlapple.Mod06Fit[,"sigma"])
min(Dlapple.Mod06Fit[,"sigma"])
mean(Dlapple.Mod07Fit[,"sd"])
var(Dlapple.Mod07Fit[,"sd"])
max(Dlapple.Mod07Fit[,"sd"])
min(Dlapple.Mod07Fit[,"sd"])
mean(Dlapple.Mod07vs06ratio)
var(Dlapple.Mod07vs06ratio)
max(Dlapple.Mod07vs06ratio)
min(Dlapple.Mod07vs06ratio)

#latex code:
Dlapple.latex <- paste("Apple & & ", ground(var(Dlapple),
  digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlapple),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox(Dlapple^2, lag=1)[1,"P-val"],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox(Dlapple^2, lag=1)[1,"Q-stat"],
  digits=2, as.character=TRUE), "}$ & ",
  length(Dlapple), " & ", length(Dlapple.where.zeros), " & ",
  ground(length(Dlapple.where.zeros)/length(Dlapple), digits=3,
  as.character=TRUE), " \\", sep="")


###########################################################
##7 EKORNES RETURNS
###########################################################

mekornes <- read.csv("ekornes.csv")
ekornes <- zoo(mekornes[,"Adj.Close"],
  order.by=as.Date(mekornes[,1], "%Y-%m-%d"))
Dlekornes <- diff(log(ekornes))*100

#zeros replaced by small non-zero value:
Dlekornes.where.zeros <- which( Dlekornes == 0)
Dlekornes.zero.eps <- min(abs(Dlekornes[-Dlekornes.where.zeros]))
Dlekornesadj <- Dlekornes
Dlekornesadj[Dlekornes.where.zeros] <- Dlekornes.zero.eps

#models, zeros replaced:
Dlekornes.Mod01 <- lgarch(Dlekornesadj, method="cex2")
Dlekornes.Mod01Fit <- fitted(Dlekornes.Mod01, verbose=TRUE)
Dlekornes.Mod01LogL0 <- logLik(Dlekornes.Mod01)/length(Dlekornesadj)
Dlekornes.Mod01LogL <- sum(dnorm(Dlekornes[Dlekornes!=0],
  sd=Dlekornes.Mod01Fit[Dlekornes!=0,1], log=TRUE))/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod01AR <- gBox(Dlekornes.Mod01Fit[,"zhat"], lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod01ARCH <- gBox(Dlekornes.Mod01Fit[,"zhat"]^2, lag=12,
  zeros.as.na=FALSE)

#cex2 models, zeros kept:
Dlekornes.Mod02 <- lgarch(Dlekornes, method="cex2")
Dlekornes.Mod02Fit <- fitted(Dlekornes.Mod02, verbose=TRUE)
Dlekornes.Mod02LogL <- logLik(Dlekornes.Mod02)/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod02AR <- gBox(Dlekornes.Mod02Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlekornes.Mod02ARCH <- gBox(Dlekornes.Mod02Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlekornes.Mod02etahat <- residuals(Dlekornes.Mod02)
Dlekornes.Mod02VCOV <- vcovhat(coef(Dlekornes.Mod02), "cex2",
  Dlekornes.Mod02etahat)
Dlekornes.Mod02SEs <- sqrt(diag(Dlekornes.Mod02VCOV))

#Francq et al. QMLE:
Dlekornes.Mod03 <- lgarchf.est(as.numeric(Dlekornes))
Dlekornes.Mod03Fit <- lgarchf.fit(as.numeric(Dlekornes.Mod03$par),
  Dlekornes.Mod03$aux)
Dlekornes.Mod03Fit <- zoo(Dlekornes.Mod03Fit,
  order.by=index(Dlekornes))
Dlekornes.Mod03LogL0 <- Dlekornes.Mod03$objective/length(Dlekornes)
Dlekornes.Mod03LogL <- sum(dnorm(Dlekornes[Dlekornes!=0],
  sd=Dlekornes.Mod03Fit[Dlekornes!=0,1], log=TRUE))/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod03AR <- gBox(Dlekornes.Mod03Fit[,"z"], lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod03ARCH <- gBox(Dlekornes.Mod03Fit[,"z"]^2, lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod03etahat <- as.numeric(Dlekornes.Mod03Fit[,"z"])
Dlekornes.Mod03VCOV <- vcovhat(Dlekornes.Mod03$par, "standard",
  Dlekornes.Mod03etahat)
Dlekornes.Mod03SEs <- sqrt(diag(Dlekornes.Mod03VCOV))

#Gaussian ARMA-QMLE models, zeros kept:
Dlekornes.Mod04 <- lgarch(Dlekornes, method="ml",
  mean.correction=TRUE)
Dlekornes.Mod04Fit <- fitted(Dlekornes.Mod04, verbose=TRUE)
Dlekornes.Mod04LogL <- logLik(Dlekornes.Mod04)/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod04AR <- gBox(Dlekornes.Mod04Fit[,"zhat"], lag=12,
  zeros.as.na=TRUE)
Dlekornes.Mod04ARCH <- gBox(Dlekornes.Mod04Fit[,"zhat"]^2, lag=12,
  zeros.as.na=TRUE)
Dlekornes.Mod04etahat <- residuals(Dlekornes.Mod04)
Dlekornes.Mod04VCOV <- vcovhat(coef(Dlekornes.Mod04), "ml",
  Dlekornes.Mod04etahat)
Dlekornes.Mod04SEs <- sqrt(diag(Dlekornes.Mod04VCOV))

#combined estimator, zeros kept:
Dlekornes.Mod05weights <- wghts(c(coredata(Dlekornes.Mod02etahat),
  coredata(Dlekornes.Mod04etahat)))
Dlekornes.Mod05omega0 <- as.numeric( t(Dlekornes.Mod05weights) %*%
  c(coef(Dlekornes.Mod02)[1],coef(Dlekornes.Mod04)[1]) )
Dlekornes.Mod05alpha0 <- as.numeric( t(Dlekornes.Mod05weights) %*%
  c(coef(Dlekornes.Mod02)[2],coef(Dlekornes.Mod04)[2]) )
Dlekornes.Mod05beta0 <- as.numeric( t(Dlekornes.Mod05weights) %*%
  c(coef(Dlekornes.Mod02)[3],coef(Dlekornes.Mod04)[3]) )
Dlekornes.Mod05mu <- as.numeric( coef(Dlekornes.Mod02)[4] + coef(Dlekornes.Mod04)[4])/2
Dlekornes.Mod05Fit <- sqrt(lgarchRecursion(as.numeric(Dlekornes),
  Dlekornes.Mod05omega0, Dlekornes.Mod05alpha0,
  Dlekornes.Mod05beta0, Dlekornes.Mod05mu)$sigma2)
Dlekornes.Mod05Fit <- zoo(Dlekornes.Mod05Fit,
  order.by=index(Dlekornes))
Dlekornes.Mod05LogL <- sum(dnorm(Dlekornes[Dlekornes!=0],
  sd=Dlekornes.Mod05Fit[Dlekornes!=0,1], log=TRUE))/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod05AVARc <- avarc(c(coredata(Dlekornes.Mod02etahat),
  coredata(Dlekornes.Mod04etahat)), coef(Dlekornes.Mod02),
  coef(Dlekornes.Mod04))
Dlekornes.Mod05SEc <- sqrt(Dlekornes.Mod05AVARc/length(Dlekornes[Dlekornes!=0]))

#ordinary GARCH:
Dlekornes.Mod06 <- garch.est(Dlekornes)
Dlekornes.Mod06Fit <- garch.fit(Dlekornes, as.numeric(Dlekornes.Mod06$par))
Dlekornes.Mod06LogL0 <- Dlekornes.Mod06$objective/length(Dlekornes)
Dlekornes.Mod06LogL <- sum(dnorm(Dlekornes[Dlekornes!=0],
  sd=Dlekornes.Mod06Fit[Dlekornes!=0,"sigma"], log=TRUE))/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod06AR <- gBox(Dlekornes.Mod06Fit[,"resid.std"], lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod06ARCH <- gBox(Dlekornes.Mod06Fit[,"resid.std"]^2, lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod06etahat <- as.numeric(Dlekornes.Mod06Fit[,"resid.std"])
Dlekornes.Mod06VCOV <- solve(-Dlekornes.Mod06$hessian)
Dlekornes.Mod06SEs <- sqrt(diag(Dlekornes.Mod06VCOV))

##zero-adjusted GARCH:
Dlekornes.Mod07 <- garch.estz(Dlekornes,
  initial.values=c(0.05, 0.02, 0.9, 0.01))
Dlekornes.Mod07Fit <- garch.fitz(Dlekornes, as.numeric(Dlekornes.Mod07$par))
Dlekornes.Mod07LogL0 <- Dlekornes.Mod07$objective/length(Dlekornes)
Dlekornes.Mod07LogL <- sum(dnorm(Dlekornes[Dlekornes!=0],
  sd=Dlekornes.Mod07Fit[Dlekornes!=0,"sd"], log=TRUE))/(length(Dlekornes)-sum(Dlekornes==0))
Dlekornes.Mod07AR <- gBox(Dlekornes.Mod07Fit[,"residuals"], lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod07ARCH <- gBox(Dlekornes.Mod07Fit[,"residuals"]^2, lag=12,
  zeros.as.na=FALSE)
Dlekornes.Mod07etahat <- as.numeric(Dlekornes.Mod07Fit[,"residuals"])
Dlekornes.Mod07VCOV <- solve(-Dlekornes.Mod07$hessian)
Dlekornes.Mod07SEs <- sqrt(diag(Dlekornes.Mod07VCOV))

#sd differences and ratios:
Dlekornes.Mod02vs03diff <- Dlekornes.Mod02Fit[,"sd"] - Dlekornes.Mod03Fit[,"sigma"]
Dlekornes.Mod02vs03ratio <- Dlekornes.Mod02Fit[,"sd"]/Dlekornes.Mod03Fit[,"sigma"]
Dlekornes.Mod02vs04diff <- Dlekornes.Mod02Fit[,"sd"] - Dlekornes.Mod04Fit[,"sd"]
Dlekornes.Mod02vs04ratio <- Dlekornes.Mod02Fit[,"sd"]/Dlekornes.Mod04Fit[,"sd"]
Dlekornes.Mod05vs03diff <- Dlekornes.Mod05Fit - Dlekornes.Mod03Fit[,"sigma"]
Dlekornes.Mod05vs03ratio <- Dlekornes.Mod05Fit/Dlekornes.Mod03Fit[,"sigma"]
Dlekornes.Mod07vs06ratio <- Dlekornes.Mod07Fit[,"sd"]/Dlekornes.Mod06Fit[,"sigma"]
#plot(Dlekornes.Mod07vs06ratio)

##desc stat of GARCH models:
mean(Dlekornes.Mod06Fit[,"sigma"])
var(Dlekornes.Mod06Fit[,"sigma"])
max(Dlekornes.Mod06Fit[,"sigma"])
min(Dlekornes.Mod06Fit[,"sigma"])
mean(Dlekornes.Mod07Fit[,"sd"])
var(Dlekornes.Mod07Fit[,"sd"])
max(Dlekornes.Mod07Fit[,"sd"])
min(Dlekornes.Mod07Fit[,"sd"])
mean(Dlekornes.Mod07vs06ratio)
var(Dlekornes.Mod07vs06ratio)
max(Dlekornes.Mod07vs06ratio)
min(Dlekornes.Mod07vs06ratio)

#latex code:
Dlekornes.latex <- paste("Ekornes & & ", ground(var(Dlekornes),
  digits=2, as.character=TRUE), " & ",
  ground(gkurtosis(Dlekornes),
  digits=2, as.character=TRUE), " & ",
  "$\\underset{[", ground(gBox(Dlekornes^2, lag=1)[1,"P-val"],
  digits=2, as.character=TRUE) , "]}{",
  ground(gBox(Dlekornes^2, lag=1)[1,"Q-stat"],
  digits=2, as.character=TRUE), "}$ & ",
  length(Dlekornes), " & ", length(Dlekornes.where.zeros), " & ",
  ground(length(Dlekornes.where.zeros)/length(Dlekornes), digits=3,
  as.character=TRUE), " \\", sep="")
  
  
###########################################################
##8 TABLES AND GRAPHS
###########################################################

rbind(Dlsp500.latex, Dlapple.latex, Dlekornes.latex)

##=================
##figure of return:

#set margin sizes:
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))

#plot
layout(matrix(1:3, 3, 1))
plot(Dlsp500, col="blue", ylab="SP500", xlab="", main="")
plot(Dlapple, col="blue", ylab="Apple", xlab="", main="")
plot(Dlekornes, col="blue", ylab="Ekornes", xlab="", main="")

#return to default margin sizes:
par(def.par)

##==============================
##figure of standard deviations:

#set margin sizes:
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))

#plot:
layout(matrix(1:3, 3, 1))
#no. 1:
tmp.sp500 <- cbind(Dlsp500.Mod02Fit[,"sd"],
  Dlsp500.Mod03Fit[,"sigma"], Dlsp500.Mod05Fit)
colnames(tmp.sp500) <- c("CEX2", "Standard", "Combined")
plot(tmp.sp500, plot.type="single",
  col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), ylab="SP500", xlab="",
  main="")
legend("topright",c("CEX2", "Standard", "Combined"),
  lwd=2, col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), bty="n", cex=1)
#no. 2:
tmp.apple <- cbind(Dlapple.Mod02Fit[,"sd"],
  Dlapple.Mod03Fit[,"sigma"], Dlapple.Mod05Fit)
colnames(tmp.apple) <- c("CEX2", "Standard", "Combined")
plot(tmp.apple, plot.type="single",
  col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), ylab="Apple", xlab="",
  main="")
legend("topright",c("CEX2", "Standard", "Combined"),
  lwd=2, col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), bty="n", cex=1)
#no. 3:
tmp.ekornes <- cbind(Dlekornes.Mod02Fit[,"sd"],
  Dlekornes.Mod03Fit[,"sigma"], Dlekornes.Mod05Fit)
colnames(tmp.ekornes) <- c("CEX2", "Standard", "Combined")
plot(tmp.ekornes, plot.type="single",
  col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), ylab="Ekornes", xlab="",
  main="")
legend("topright",c("CEX2", "Standard", "Combined"),
  lwd=2, col=c("blue", "red", "green4"),
  lty=c("solid","dashed", "dotted"), bty="n", cex=1)

#return to default margin sizes:
par(def.par)

##==============================
##figure of log-garch ratios:

#set margin sizes:
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))

#plot:
layout(matrix(1:3, 3, 1))
tmp.sp500 <- cbind(Dlsp500.Mod05vs03ratio, Dlsp500.Mod02vs03ratio)
plot(tmp.sp500, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="SP500", xlab="",
  main="", ylim=c(0.7,1.4))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("bottomleft",c("C/S", "X/S"),
  lwd=2, col=c("red", "blue"), lty=c("dashed","solid"),
  bty="n", cex=1)
tmp.apple <- cbind(Dlapple.Mod05vs03ratio, Dlapple.Mod02vs03ratio)
plot(tmp.apple, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="Apple", xlab="",
  main="", ylim=c(0.7,1.4))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("bottomleft",c("C/S", "X/S"),
  lwd=2, col=c("red", "blue"), lty=c("dashed","solid"),
  bty="n", cex=1)
tmp.ekornes <- cbind(Dlekornes.Mod05vs03ratio, Dlekornes.Mod02vs03ratio)
plot(tmp.ekornes, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="Ekornes", xlab="",
  main="", ylim=c(0.7,1.4))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("bottomleft",c("C/S", "X/S"),
  lwd=2, col=c("red", "blue"), lty=c("dashed","solid"),
  bty="n", cex=1)

#return to default margin sizes:
par(def.par)

##=====================================
##figure of log-garch and garch ratios:

#set margin sizes:
def.par <- par(no.readonly=TRUE)
par(mai=c(0.3, 0.55, 0.05, 0.05))

#plot:
layout(matrix(1:3, 3, 1))
tmp.sp500 <- cbind(Dlsp500.Mod07vs06ratio, Dlsp500.Mod02vs03ratio)
plot(tmp.sp500, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="SP500", xlab="",
  main="", ylim=c(0.9,1.3))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("topleft",c("X/S ratio (log-GARCH)", "A/S ratio (GARCH)"),
  lwd=2, col=c("blue", "red"), lty=c("solid","dashed"),
  bty="n", cex=1)
tmp.apple <- cbind(Dlapple.Mod07vs06ratio, Dlapple.Mod02vs03ratio)
plot(tmp.apple, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="Apple", xlab="",
  main="", ylim=c(0.9,1.3))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("topleft",c("X/S ratio (log-GARCH)", "A/S ratio (GARCH)"),
  lwd=2, col=c("blue", "red"), lty=c("solid","dashed"),
  bty="n", cex=1)
tmp.ekornes <- cbind(Dlekornes.Mod07vs06ratio, Dlekornes.Mod02vs03ratio)
plot(tmp.ekornes, plot.type="single", col=c("red", "blue"),
  lty=c("dashed","solid"), ylab="Ekornes", xlab="",
  main="", ylim=c(0.9,1.3))
abline(1,0, col="grey", lwd=1) #add zero-line
legend("topleft",c("X/S ratio (log-GARCH)", "A/S ratio (GARCH)"),
  lwd=2, col=c("blue", "red"), lty=c("solid","dashed"),
  bty="n", cex=1)

#return to default margin sizes:
par(def.par)
